package greenfoot;

import android.app.ActivityManager;
import android.app.ActivityManager.RunningAppProcessInfo;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.RectF;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.SurfaceHolder.Callback;
import java.util.ArrayList;
import java.util.List;

import java.lang.StackTraceElement;

/**
 * @author Upz
 */
final class GameView extends SurfaceView implements Callback {
	
	World currentWorld;
	ActivityManager am;
	
	SurfaceHolder sfh;
	Paint paint;
	Canvas canvas;
	GameFPS fps;
	boolean show = false;
	
	List<String> infoLines = new ArrayList<String>();
	List<String> extraLines = new ArrayList<String>();
	
	
	
	public GameView(Context context) {
		super(context);
		init();
	}
	
	public void init(){
		environmentInit();
		
		drawingInit();
		drawingSetings();
		
		setFocusable(true);
		setLongClickable(true);
	}
	
	//make sure Greenfoot is ready before this method is called
	public void environmentInit(){
		am = (ActivityManager)(Greenfoot.activity.getSystemService(Context.ACTIVITY_SERVICE));
		
		setOnTouchListener(new TouchManager());
		
		//currentWorld = Greenfoot.currentWorld;
		//Greenfoot.placeWorldFrame();
		//Greenfoot.setScaleRate();
	}
	
	public void drawingInit(){
		sfh = this.getHolder();		
		sfh.addCallback(this);
		
		paint = new Paint();
		paint.setStyle(Paint.Style.STROKE);
		//paint.setStrokeWidth(5f);
		paint.setColor(Color.WHITE.getAndroidColor());
		//paint.setAntiAlias(true);
		
		fps = new GameFPS();
	}
	 
	 
	//creat & start the GameThread here
	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		Greenfoot.viewCreated();
	}
	
	private void drawingSetings(){
		//sfh.setType(SurfaceHolder.SURFACE_TYPE_HARDWARE);
		//sfh.setType(SurfaceHolder.SURFACE_TYPE_GPU);

		//sfh.setFormat(PixelFormat.RGB_565);
		
		//this.setWillNotCacheDrawing(false);
	}
	
	
	public void draw() {
		try {
			canvas = sfh.lockCanvas();
			if (canvas != null) {
				//clear
				canvas.drawRGB(120, 120, 120);
				
				canvas.save();
				//RectF frameRect = new RectF(manager.getWorldFrameLeft(), manager.getWorldFrameTop(), manager.getWorldFrameRight(), manager.getWorldFrameBottom());
				//canvas.scale(0.999999f, 0.999999f, Greenfoot.worldFrameX, Greenfoot.worldFrameY);
				//canvas.drawRect(frameRect, paint);
				canvas.translate(Greenfoot.worldFrameX, Greenfoot.worldFrameY);
				if(Greenfoot.scaleRate != 1)
					canvas.scale(Greenfoot.scaleRate, Greenfoot.scaleRate);
				canvas.translate(-Greenfoot.getWorldWidthPx() / 2, -Greenfoot.getWorldHeightPx() / 2);
				RectF rect = new RectF(0, 0, Greenfoot.getWorldWidthPx(), Greenfoot.getWorldHeightPx());
				canvas.clipRect(rect);
				
				//draw the world
				canvas.drawBitmap(currentWorld.background.bmp, 0, 0, paint);
				if(!currentWorld.actorList.isEmpty())
					for(int i = 0; i < currentWorld.paintList.size(); i++){
					Actor actor = currentWorld.paintList.get(i);
					canvas.save();
					if(actor.rotated)
						canvas.rotate(actor.rotation, actor.x * currentWorld.cellSize, actor.y * currentWorld.cellSize);
					if(actor.scaled)
						canvas.scale(actor.scaleX, actor.scaleY, actor.x, actor.y);
						canvas.translate(actor.x * currentWorld.cellSize - actor.image.width / 2f, actor.y * currentWorld.cellSize - actor.image.height / 2f);
						canvas.clipRect(new RectF(0, 0,actor.image.width, actor.image.height));
					actor.draw(canvas);
					canvas.restore();
				}
				canvas.drawRect(rect, paint);
				float sw = paint.getStrokeWidth();
				paint.setStrokeWidth(1);
				canvas.drawPoint(Greenfoot.getTouchX() * currentWorld.cellSize, Greenfoot.getTouchY() * currentWorld.cellSize, paint);
				paint.setStrokeWidth(sw);
				canvas.restore();
				
				
				//draw some notes
				if(show){
					addViewInfo();
					addPointInfo();
					addCountInfo();
					addExtraInfo();
				}
				//*/
				if(!Greenfoot.errLines.isEmpty())
				addExceptionInfo();
				if(Greenfoot.errLines.isEmpty() && !Greenfoot.Running){
					infoLines.add("");
					infoLines.add("" + Greenfoot.Running);
					infoLines.add("Warnning : ");
					infoLines.add("current world is not started.");
					infoLines.add("have you added line");
					infoLines.add("  Greenfoot.start();");
					infoLines.add("    at the end of your world's constructor?");
				}
				
				sw = paint.getStrokeWidth();
				int color = paint.getColor();
				paint.setStrokeWidth(5);
			    paint.setColor(Color.BLACK.getAndroidColor());
				drawInfo(canvas);
				paint.setStrokeWidth(sw);
				paint.setColor(color);
				drawInfo(canvas);
				
				clearInfo();
				fps.count();
			}
		} catch (Exception e) {
			
		} finally {
			if (canvas != null)
				sfh.unlockCanvasAndPost(canvas);
		}
	}
	
	public void clearInfo(){
		infoLines.clear();
		//errLines.clear();
		extraLines.clear();
	}
	
	public void drawInfo(Canvas c){
		for(int i = 0; i < infoLines.size(); i++){
			c.drawText(infoLines.get(i), 30, 16 * i, paint);
		}
	}
	
	public void addViewInfo(){
		infoLines.add("");
		infoLines.add("View Infomation : ");
		infoLines.add("ScreenView : " + this.getWidth() + "  *  " + this.getHeight());
		infoLines.add("WorldView : " + Greenfoot.getWorldWidthPx() + "  *  " + Greenfoot.getWorldHeightPx());
		infoLines.add("WorldScaleRate : " + Greenfoot.scaleRate);
	}
	
	public void addPointInfo(){
		infoLines.add("");
		infoLines.add("Point Infomation : ");
		infoLines.add("TouchPoint : " + (int)Greenfoot.getTouchX() + "," + (int)Greenfoot.getTouchY() + "    (" + Greenfoot.touchManager.pointers.size() + ")");
		infoLines.add("isTouching : " + Greenfoot.touchManager.currentMouseInfo.isTouching);
		infoLines.add("isDragging : " + Greenfoot.touchManager.currentMouseInfo.isDragged);
		infoLines.add("lastKey : " + Greenfoot.touchManager.currentKeyInfo.lastKey + " " + Greenfoot.touchManager.bufferedKeyInfo.lastKey + Greenfoot.touchManager.bufferedKeyInfo.isKeyDown("space"));
	}
	
	int memoryrefresh = 0;
	int mem = 0;
	public void addCountInfo(){
		infoLines.add("");
		infoLines.add("Count Infomation : ");
		infoLines.add("ActorsNum : " + currentWorld.numberOfObjects() + "( " + currentWorld.actList.size() + " , " + currentWorld.paintList.size() + " )");
		infoLines.add("FPS : " + fps.fps);
		//infoLines.add(android.os.Environment.getExternalStorageDirectory().toString());
		
		/*
		infoLines.add("MemoryUse : ");
		if(memoryrefresh > 60){
			List<RunningAppProcessInfo> pi = am.getRunningAppProcesses();
			for(int i = 0; i < pi.size(); i++){
				if(pi.get(i).processName.equals("com.Runner")){
					int[] pid = {pi.get(i).pid};
					mem = am.getProcessMemoryInfo(pid)[0].dalvikPrivateDirty;
				}
			}
			memoryrefresh = 0;
		}
		memoryrefresh++;
		infoLines.add("" + mem + "Kb");
		//*/
	}
	
	public void addExceptionInfo(){
		infoLines.add("");
		infoLines.add("");
		infoLines.add("Exception Infomation : ");
		if(!Greenfoot.errLines.isEmpty())
		for(int i = 0; i < Greenfoot.errLines.size(); i++)
		infoLines.add(Greenfoot.errLines.get(i));
	}
	

	
	public void addExtraInfo(){
		infoLines.add("");
		infoLines.add("Extra Infomation : ");
		if(!extraLines.isEmpty())
			for(int i = 0; i < extraLines.size(); i++)
				infoLines.add(extraLines.get(i));
	}
	
	public void addExtraLines(String s){
		extraLines.add(s);
	}
	
	
	
	
	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
	}
	
	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		Greenfoot.destroyThread();
	}
}
