import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;
import java.awt.Point;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Color;
/**
 * A DropDownList
 * 
 * @author Taylor Born
 * @version 1.0
 * <p>
 * March 2011
 */
public class ComboBox  extends Interface
{
    static Font font = new Font("New Times Roman", Font.PLAIN, 12);
    
    private ArrayList<String> items = new ArrayList<String>();
    private int index = -1;
    private Point home;
    private Point lastMouse = new Point(-25, -25);
    private boolean selecting = false;
    
    private boolean changed = false;

    /**
     * Create a new DropDownList.
     * @param items The contents of the DropDownList.
     * @param index The initial selected item index.
     */
    public ComboBox(ArrayList<String> items, int index)
    {
        this.items = items;
        this.index = index;
    }
    
    public void act() 
    {
        MouseInfo mouse = Greenfoot.getMouseInfo();
        if (Greenfoot.mouseDragged(null))
            lastMouse = new Point(mouse.getX(), mouse.getY());
        
        int height = 12;
        if (selecting)
            height += items.size() * 12;
        
        int width = 0;
        
        setLocation(getX(), (int)home.y + (height - 12) / 2);
        Point offsetMouse = new Point((int)(lastMouse).x - (getX() - (width / 2)), (int)lastMouse.y - (getY() - (height / 2)));
        
        boolean clicked = Greenfoot.mouseClicked(null);
        if (Greenfoot.mouseClicked(this) && offsetMouse.x > 0 && offsetMouse.x < width && offsetMouse.y > 0 && offsetMouse.y < 12)
            selecting = !selecting;
        
        GreenfootImage pic = new GreenfootImage(width + 1, height + 1);
        pic.setFont(font);
        
        pic.setColor(Color.WHITE);
        pic.fill();
        pic.setColor(Color.BLACK);
        pic.drawRect(0, 0, width, 12);
        if (selecting)
            pic.setColor(Color.GRAY);
        if (index > -1 && index < items.size())
            pic.drawString(items.get(index), 2, 11);
        
        if (selecting)
            for (int i = 0; i < items.size(); i++)
            {
                if (offsetMouse.x > 0 && offsetMouse.x < width && offsetMouse.y > 12 * (i + 1) && offsetMouse.y <= 12 * (i + 2))
                {
                    pic.setColor(Color.CYAN);
                    pic.fillRect(0, 12 * (i + 1), width, 12);
                    if (clicked)
                    {
                        if (index != i)
                            changed = true;
                        index = i;
                        selecting = false;
                    }
                }
                pic.setColor(Color.BLACK);
                pic.drawRect(0, 12 * (i + 1), width, 12);
                pic.drawString(items.get(i), 2, (i + 1) * 12 + 11);
            }
        
        if (!Greenfoot.mouseClicked(this) && Greenfoot.mouseClicked(null))
            selecting = false;
        
        setImage(pic);
    }
    
    /**
     * @return String for the item that is selected. Null if none selected.
     */
    public String getSelected()
    {
        if (index == -1)
            return null;
        return items.get(index);
    }
    
    /**
     * @return The index of the item that is selected. -1 if none selected.
     */
    public int getIndex()
    {
        return index;
    }
    
    /**
     * @return The contents in this list.
     */
    public ArrayList<String> getList()
    {
        return items;
    }
    
    /**
     * The action listener for this DropDownList.
     * @return Whether a new item has been selected.
     */
    public boolean hasChanged()
    {
        boolean c = changed;
        changed = false;
        return c;
    }
    
    public void addedToWorld(World world)
    {
        home = new Point(getX(), getY());
    }
}